#include <malloc.h>
#include	"curses.ext"

/*	newwin.c
 *
 *	Allocate space for and set up defaults for a new window.
 *	Sets internal flags depending upon size of window.
 *	A window created with newwin() has its own memory for
 *	storage.  A window created with subwin() shares its
 *	memory with its parent window (changing a character in a subwindow
 *	changes the corresponding character in the parent window -since
 *	they are one and the same).  Subwindows can overlap.
 */

extern WINDOW *makenew();


WINDOW *newwin( num_lines, num_cols, begy, begx )
INT num_lines;		/* number of lines in window */
INT num_cols;		/* number of columns in window */
INT begy;			/* beginning line */
INT begx;			/* beginning column */
	{
	REG WINDOW  *win;
	REG INT     i;
	REG SHORT *sp;
	INT    j;
	SHORT *end, _space;

	if (num_lines == 0)
		num_lines = LINES - begy;
	if (num_cols == 0)
		num_cols = COLS - begx;
	if ((win = makenew( num_lines, num_cols, begy, begx)) == NULL)
		return( ERR );

	for (i = 0; i < num_lines; ++i)
		{
		if ((win->_y[i] = calloc( num_cols, sizeof (short int) )) == NULL)
			{
			for (j = 0; j < i; ++j)
				free( win->_y[j] );
			free( win->_firstch );
			free( win->_lastch );
			free( win->_y );
			free( win );
			return( ERR );
			}
		}

	win->_nextp = 0;
	win->_orig = (struct _st *)win;

	_space = (_MTY_SPACE | _ATTR_NORMAL);

	for ( i = 0; i < win->_maxy; ++i )	/* set window to blanks */
		{
		for (sp = win->_y[i], end = sp + win->_maxx; sp < end; ++sp)
			*sp = _space;
		}
	return( win );
	}


WINDOW *subwin( orig, num_lines, num_cols, begy, begx )
REG WINDOW *orig;	/* parent window pointer */
INT num_lines;			/* number of lines in window */
INT num_cols;			/* number of columns in window */
INT begy;				/* beginning line */
INT begx;				/* beginning column */
	{
	REG INT i;
	REG WINDOW *win;
	INT j, k;

	/*
	 * insure window fits inside the original one
	 */

	if (begy < orig->_begy || begx < orig->_begx
	  || begy + num_lines > orig->_maxy || begx + num_cols > orig->_maxx)
			return( ERR );

	if (num_lines == 0)
		num_lines = orig->_maxy - orig->_begy - begy;
	if (num_cols == 0)
		num_cols = orig->_maxx - orig->_begx - begx;
	if ((win = makenew( num_lines, num_cols, begy, begx )) == NULL)
		return( ERR );

	j = orig->_begy + begy;
	k = orig->_begx + begx;
	for (i = 0; i < num_lines; ++i)
		win->_y[i] = &orig->_y[j++][k];
	win->_flags = _SUBWIN;

	win->_nextp = orig->_nextp;
	orig->_nextp = (struct _st *)win;
	win->_orig = (struct _st *)orig;

	return( win );
	}


/*
 *		This routine sets up a window buffer and returns a pointer to it.
 */
static WINDOW *makenew( num_lines, num_cols, begy, begx )
INT num_lines;		/* number of lines in window */
INT num_cols;		/* number of columns in window */
INT begy;			/* beginning line */
INT begx;			/* beginning column */
	{
	REG INT i;
	REG WINDOW *win;

	if ((win = (WINDOW *) malloc( sizeof( WINDOW ) )) == NULL)
		return( NULL );

	if ((win->_y = (short int **) calloc( num_lines, sizeof( short int ** ) )) == NULL)
		{
		free( win );
		return( (WINDOW *) NULL );
		}

	if ((win->_firstch = (short *) calloc( num_lines, sizeof( short * ) )) == NULL)
		{
		free( win->_y );
		free( win );
		return( (WINDOW *) NULL );
		}

	if ((win->_lastch = (short *) calloc( num_lines, sizeof( short * ) )) == NULL)
		{
		free( win->_y );
		free( win->_firstch );
		free( win );
		return( (WINDOW *) NULL );
		}

	win->_cury = win->_curx = 0;
	win->_clear = 0;
	win->_maxy = num_lines;
	win->_maxx = num_cols;
	win->_begy = begy;
	win->_begx = begx;
	win->_scroll = FALSE;
	win->_flags = _NORMAL;
	win->_attr = _ATTR_NORMAL;
	for (i = 0; i < num_lines; ++i)
		win->_firstch[i] = win->_lastch[i] = _NOCHANGE;
	if (begx + num_cols == COLS)
		{
		win->_flags |= _ENDLINE;
		if (begx == 0 && num_lines == LINES && begy == 0)
			win->_flags |= _FULLWIN;
		if ( ( begy + num_lines ) == LINES )
			win->_flags |= _SCROLLWIN;
		}

	return( win );
	}
